@@ -575,71 +575,239 @@ class TestConvertDtype(TestCase):
         spec = DatasetSpec('an example dataset', RefSpec(reftype='object', target_type='int'), name='data')
         self.assertTupleEqual(ObjectMapper.convert_dtype(spec, None), (None, 'object'))
 
-    def test_convert_higher_precision(self):
-        """Test that passing a data type with a precision <= specified returns the higher precision type"""
+    # do full matrix test of given value x and spec y, what does convert_dtype return?
+    def test_convert_to_64bit_spec(self):
+        """
+        Test that if given any value for a spec with a 64-bit dtype, convert_dtype will convert to the spec type.
+        Also test that if the given value is not the same as the spec, convert_dtype raises a warning.
+        """
         spec_type = 'float64'
-        value_types = ['float', 'float32', 'double', 'float64']
-        self.convert_higher_precision_helper(spec_type, value_types)
+        value_types = ['double', 'float64']
+        self._test_convert_alias(spec_type, value_types)
+
+        spec_type = 'float64'
+        value_types = ['float', 'float32', 'long', 'int64', 'int', 'int32', 'int16', 'int8', 'uint64', 'uint',
+                       'uint32', 'uint16', 'uint8', 'bool']
+        self._test_convert_higher_precision_helper(spec_type, value_types)
 
         spec_type = 'int64'
-        value_types = ['long', 'int64', 'uint64', 'int', 'int32', 'int16', 'int8']
-        self.convert_higher_precision_helper(spec_type, value_types)
+        value_types = ['long', 'int64']
+        self._test_convert_alias(spec_type, value_types)
 
-        spec_type = 'int32'
-        value_types = ['int32', 'int16', 'int8']
-        self.convert_higher_precision_helper(spec_type, value_types)
+        spec_type = 'int64'
+        value_types = ['double', 'float64', 'float', 'float32', 'int', 'int32', 'int16', 'int8', 'uint64', 'uint',
+                       'uint32', 'uint16', 'uint8', 'bool']
+        self._test_convert_higher_precision_helper(spec_type, value_types)
 
-        spec_type = 'int16'
-        value_types = ['int16', 'int8']
-        self.convert_higher_precision_helper(spec_type, value_types)
-
-        spec_type = 'uint32'
-        value_types = ['uint32', 'uint16', 'uint8']
-        self.convert_higher_precision_helper(spec_type, value_types)
-
-    def convert_higher_precision_helper(self, spec_type, value_types):
-        data = 2
-        spec = DatasetSpec('an example dataset', spec_type, name='data')
-        match = (np.dtype(spec_type).type(data), np.dtype(spec_type))
-        for dtype in value_types:
-            value = np.dtype(dtype).type(data)
-            with self.subTest(dtype=dtype):
-                ret = ObjectMapper.convert_dtype(spec, value)
-                self.assertTupleEqual(ret, match)
-                self.assertIs(ret[0].dtype, match[1])
-
-    def test_keep_higher_precision(self):
-        """Test that passing a data type with a precision >= specified return the given type"""
-        spec_type = 'float'
-        value_types = ['double', 'float64']
-        self.keep_higher_precision_helper(spec_type, value_types)
-
-        spec_type = 'int'
-        value_types = ['int64']
-        self.keep_higher_precision_helper(spec_type, value_types)
-
-        spec_type = 'int8'
-        value_types = ['long', 'int64', 'int', 'int32', 'int16']
-        self.keep_higher_precision_helper(spec_type, value_types)
-
-        spec_type = 'uint'
+        spec_type = 'uint64'
         value_types = ['uint64']
-        self.keep_higher_precision_helper(spec_type, value_types)
+        self._test_convert_alias(spec_type, value_types)
 
-        spec_type = 'uint8'
-        value_types = ['uint64', 'uint32', 'uint', 'uint16']
-        self.keep_higher_precision_helper(spec_type, value_types)
+        spec_type = 'uint64'
+        value_types = ['double', 'float64', 'float', 'float32', 'long', 'int64', 'int', 'int32', 'int16', 'int8',
+                       'uint', 'uint32', 'uint16', 'uint8', 'bool']
+        self._test_convert_higher_precision_helper(spec_type, value_types)
 
-    def keep_higher_precision_helper(self, spec_type, value_types):
-        data = 2
+    def test_convert_to_float32_spec(self):
+        """Test conversion of various types to float32.
+        If given a value with precision > float32 and float base type, convert_dtype will keep the higher precision.
+        If given a value with 64-bit precision and different base type, convert_dtype will convert to float64.
+        If given a value that is float32, convert_dtype will convert to float32.
+        If given a value with precision <= float32, convert_dtype will convert to float32 and raise a warning.
+        """
+        spec_type = 'float32'
+        value_types = ['double', 'float64']
+        self._test_keep_higher_precision_helper(spec_type, value_types)
+
+        value_types = ['long', 'int64', 'uint64']
+        expected_type = 'float64'
+        self._test_change_basetype_helper(spec_type, value_types, expected_type)
+
+        value_types = ['float', 'float32']
+        self._test_convert_alias(spec_type, value_types)
+
+        value_types = ['int', 'int32', 'int16', 'int8', 'uint', 'uint32', 'uint16', 'uint8', 'bool']
+        self._test_convert_higher_precision_helper(spec_type, value_types)
+
+    def test_convert_to_int32_spec(self):
+        """Test conversion of various types to int32.
+        If given a value with precision > int32 and int base type, convert_dtype will keep the higher precision.
+        If given a value with 64-bit precision and different base type, convert_dtype will convert to int64.
+        If given a value that is int32, convert_dtype will convert to int32.
+        If given a value with precision <= int32, convert_dtype will convert to int32 and raise a warning.
+        """
+        spec_type = 'int32'
+        value_types = ['int64', 'long']
+        self._test_keep_higher_precision_helper(spec_type, value_types)
+
+        value_types = ['double', 'float64', 'uint64']
+        expected_type = 'int64'
+        self._test_change_basetype_helper(spec_type, value_types, expected_type)
+
+        value_types = ['int', 'int32']
+        self._test_convert_alias(spec_type, value_types)
+
+        value_types = ['float', 'float32', 'int16', 'int8', 'uint', 'uint32', 'uint16', 'uint8', 'bool']
+        self._test_convert_higher_precision_helper(spec_type, value_types)
+
+    def test_convert_to_uint32_spec(self):
+        """Test conversion of various types to uint32.
+        If given a value with precision > uint32 and uint base type, convert_dtype will keep the higher precision.
+        If given a value with 64-bit precision and different base type, convert_dtype will convert to uint64.
+        If given a value that is uint32, convert_dtype will convert to uint32.
+        If given a value with precision <= uint32, convert_dtype will convert to uint32 and raise a warning.
+        """
+        spec_type = 'uint32'
+        value_types = ['uint64']
+        self._test_keep_higher_precision_helper(spec_type, value_types)
+
+        value_types = ['double', 'float64', 'long', 'int64']
+        expected_type = 'uint64'
+        self._test_change_basetype_helper(spec_type, value_types, expected_type)
+
+        value_types = ['uint', 'uint32']
+        self._test_convert_alias(spec_type, value_types)
+
+        value_types = ['float', 'float32', 'int', 'int32', 'int16', 'int8', 'uint16', 'uint8', 'bool']
+        self._test_convert_higher_precision_helper(spec_type, value_types)
+
+    def test_convert_to_int16_spec(self):
+        """Test conversion of various types to int16.
+        If given a value with precision > int16 and int base type, convert_dtype will keep the higher precision.
+        If given a value with 64-bit precision and different base type, convert_dtype will convert to int64.
+        If given a value with 32-bit precision and different base type, convert_dtype will convert to int32.
+        If given a value that is int16, convert_dtype will convert to int16.
+        If given a value with precision <= int16, convert_dtype will convert to int16 and raise a warning.
+        """
+        spec_type = 'int16'
+        value_types = ['long', 'int64', 'int', 'int32']
+        self._test_keep_higher_precision_helper(spec_type, value_types)
+
+        value_types = ['double', 'float64', 'uint64']
+        expected_type = 'int64'
+        self._test_change_basetype_helper(spec_type, value_types, expected_type)
+
+        value_types = ['float', 'float32', 'uint', 'uint32']
+        expected_type = 'int32'
+        self._test_change_basetype_helper(spec_type, value_types, expected_type)
+
+        value_types = ['int16']
+        self._test_convert_alias(spec_type, value_types)
+
+        value_types = ['int8', 'uint16', 'uint8', 'bool']
+        self._test_convert_higher_precision_helper(spec_type, value_types)
+
+    def test_convert_to_uint16_spec(self):
+        """Test conversion of various types to uint16.
+        If given a value with precision > uint16 and uint base type, convert_dtype will keep the higher precision.
+        If given a value with 64-bit precision and different base type, convert_dtype will convert to uint64.
+        If given a value with 32-bit precision and different base type, convert_dtype will convert to uint32.
+        If given a value that is uint16, convert_dtype will convert to uint16.
+        If given a value with precision <= uint16, convert_dtype will convert to uint16 and raise a warning.
+        """
+        spec_type = 'uint16'
+        value_types = ['uint64', 'uint', 'uint32']
+        self._test_keep_higher_precision_helper(spec_type, value_types)
+
+        value_types = ['double', 'float64', 'long', 'int64']
+        expected_type = 'uint64'
+        self._test_change_basetype_helper(spec_type, value_types, expected_type)
+
+        value_types = ['float', 'float32', 'int', 'int32']
+        expected_type = 'uint32'
+        self._test_change_basetype_helper(spec_type, value_types, expected_type)
+
+        value_types = ['uint16']
+        self._test_convert_alias(spec_type, value_types)
+
+        value_types = ['int16', 'int8', 'uint8', 'bool']
+        self._test_convert_higher_precision_helper(spec_type, value_types)
+
+    def test_convert_to_bool_spec(self):
+        """Test conversion of various types to bool.
+        If given a value with type bool, convert_dtype will convert to bool.
+        If given a value with type int8/uint8, convert_dtype will convert to bool and raise a warning.
+        Otherwise, convert_dtype will raise an error.
+        """
+        spec_type = 'bool'
+        value_types = ['bool']
+        self._test_convert_alias(spec_type, value_types)
+
+        value_types = ['uint8', 'int8']
+        self._test_convert_higher_precision_helper(spec_type, value_types)
+
+        value_types = ['double', 'float64', 'float', 'float32', 'long', 'int64', 'int', 'int32', 'int16', 'uint64',
+                       'uint', 'uint32', 'uint16']
+        self._test_convert_mismatch_helper(spec_type, value_types)
+
+    def _get_type(self, type_str):
+        return ObjectMapper._ObjectMapper__dtypes[type_str]  # apply ObjectMapper mapping string to dtype
+
+    def _test_convert_alias(self, spec_type, value_types):
+        data = 1
         spec = DatasetSpec('an example dataset', spec_type, name='data')
+        match = (self._get_type(spec_type)(data), self._get_type(spec_type))
         for dtype in value_types:
-            value = np.dtype(dtype).type(data)
-            match = (value, np.dtype(dtype))
+            value = self._get_type(dtype)(data)  # convert data to given dtype
             with self.subTest(dtype=dtype):
                 ret = ObjectMapper.convert_dtype(spec, value)
                 self.assertTupleEqual(ret, match)
-                self.assertIs(ret[0].dtype, match[1])
+                self.assertIs(ret[0].dtype.type, match[1])
+
+    def _test_convert_higher_precision_helper(self, spec_type, value_types):
+        data = 1
+        spec = DatasetSpec('an example dataset', spec_type, name='data')
+        match = (self._get_type(spec_type)(data), self._get_type(spec_type))
+        for dtype in value_types:
+            value = self._get_type(dtype)(data)  # convert data to given dtype
+            with self.subTest(dtype=dtype):
+                s = np.dtype(self._get_type(spec_type))
+                g = np.dtype(self._get_type(dtype))
+                msg = 'Value with data type %s is being converted to data type %s as specified.' % (g.name, s.name)
+                with self.assertWarnsWith(UserWarning, msg):
+                    ret = ObjectMapper.convert_dtype(spec, value)
+                self.assertTupleEqual(ret, match)
+                self.assertIs(ret[0].dtype.type, match[1])
+
+    def _test_keep_higher_precision_helper(self, spec_type, value_types):
+        data = 1
+        spec = DatasetSpec('an example dataset', spec_type, name='data')
+        for dtype in value_types:
+            value = self._get_type(dtype)(data)
+            match = (value, self._get_type(dtype))
+            with self.subTest(dtype=dtype):
+                ret = ObjectMapper.convert_dtype(spec, value)
+                self.assertTupleEqual(ret, match)
+                self.assertIs(ret[0].dtype.type, match[1])
+
+    def _test_change_basetype_helper(self, spec_type, value_types, exp_type):
+        data = 1
+        spec = DatasetSpec('an example dataset', spec_type, name='data')
+        match = (self._get_type(exp_type)(data), self._get_type(exp_type))
+        for dtype in value_types:
+            value = self._get_type(dtype)(data)  # convert data to given dtype
+            with self.subTest(dtype=dtype):
+                s = np.dtype(self._get_type(spec_type))
+                e = np.dtype(self._get_type(exp_type))
+                g = np.dtype(self._get_type(dtype))
+                msg = ('Value with data type %s is being converted to data type %s (min specification: %s).'
+                       % (g.name, e.name, s.name))
+                with self.assertWarnsWith(UserWarning, msg):
+                    ret = ObjectMapper.convert_dtype(spec, value)
+                self.assertTupleEqual(ret, match)
+                self.assertIs(ret[0].dtype.type, match[1])
+
+    def _test_convert_mismatch_helper(self, spec_type, value_types):
+        data = 1
+        spec = DatasetSpec('an example dataset', spec_type, name='data')
+        for dtype in value_types:
+            value = self._get_type(dtype)(data)  # convert data to given dtype
+            with self.subTest(dtype=dtype):
+                s = np.dtype(self._get_type(spec_type))
+                g = np.dtype(self._get_type(dtype))
+                msg = "expected %s, received %s - must supply %s" % (s.name, g.name, s.name)
+                with self.assertRaisesWith(ValueError, msg):
+                    ObjectMapper.convert_dtype(spec, value)
 
     def test_no_spec(self):
         spec_type = None
