@@ -65,6 +65,25 @@ class TestSpec(TestCase):
             nsstr = file.read()
             self.assertEqual(nsstr, match_str)
 
+    def _test_namespace_file(self):
+        with open(self.namespace_path, 'r') as file:
+            match_str = \
+"""namespaces:
+- author: foo
+  contact: foo@bar.com
+  date: '%s'
+  doc: mydoc
+  full_name: My Laboratory
+  name: mylab
+  schema:
+  - doc: Extensions for my lab
+    source: mylab.extensions.yaml
+    title: Extensions for my lab
+  version: 0.0.1
+""" % self.date.isoformat()  # noqa: E128
+            nsstr = file.read()
+            self.assertEqual(nsstr, match_str)
+
 
 class TestNamespaceBuilder(TestSpec):
     NS_NAME = 'test_ns'
@@ -88,25 +107,6 @@ class TestNamespaceBuilder(TestSpec):
         self._test_namespace_file()
         self._test_extensions_file()
 
-    def _test_namespace_file(self):
-        with open(self.namespace_path, 'r') as file:
-            match_str = \
-"""namespaces:
-- author: foo
-  contact: foo@bar.com
-  date: '%s'
-  doc: mydoc
-  full_name: My Laboratory
-  name: mylab
-  schema:
-  - doc: Extensions for my lab
-    source: mylab.extensions.yaml
-    title: Extensions for my lab
-  version: 0.0.1
-""" % self.date.isoformat()  # noqa: E128
-            nsstr = file.read()
-            self.assertEqual(nsstr, match_str)
-
     def test_read_namespace(self):
         ns_catalog = NamespaceCatalog()
         ns_catalog.load_namespaces(self.namespace_path, resolve=True)
@@ -137,6 +137,18 @@ class TestNamespaceBuilder(TestSpec):
                                      'source': 'mylab.extensions.yaml',
                                      'title': 'Extensions for my lab'})
 
+    def test_missing_version(self):
+        """Test that creating a namespace builder without a version raises an error."""
+        msg = "Namespace '%s' missing key 'version'. Please specify a version for the extension." % self.ns_name
+        with self.assertRaisesWith(ValueError, msg):
+            self.ns_builder = NamespaceBuilder(doc="mydoc",
+                                               name=self.ns_name,
+                                               full_name="My Laboratory",
+                                               author="foo",
+                                               contact="foo@bar.com",
+                                               namespace_cls=SpecNamespace,
+                                               date=self.date)
+
 
 class TestYAMLSpecWrite(TestSpec):
 
@@ -167,29 +179,11 @@ class TestYAMLSpecWrite(TestSpec):
     def test_get_name(self):
         self.assertEqual(self.ns_name, self.ns_builder.name)
 
-    def _test_namespace_file(self):
-        with open(self.namespace_path, 'r') as file:
-            match_str = \
-"""namespaces:
-- author: foo
-  contact: foo@bar.com
-  date: '%s'
-  doc: mydoc
-  full_name: My Laboratory
-  name: mylab
-  schema:
-  - doc: Extensions for my lab
-    source: mylab.extensions.yaml
-    title: Extensions for my lab
-  version: 0.0.1
-""" % self.date.isoformat()  # noqa: E128
-            nsstr = file.read()
-            self.assertEqual(nsstr, match_str)
-
 
 class TestExportSpec(TestSpec):
 
     def test_export(self):
+        """Test that export_spec writes the correct files."""
         export_spec(self.ns_builder, self.data_types, '.')
         self._test_namespace_file()
         self._test_extensions_file()
@@ -201,8 +195,7 @@ class TestExportSpec(TestSpec):
             os.remove(self.namespace_path)
 
     def _test_namespace_file(self):
-        with open(self.namespace_path, 'r') as nsfile:
-            nsstr = nsfile.read()
+        with open(self.namespace_path, 'r') as file:
             match_str = \
 """namespaces:
 - author: foo
@@ -215,13 +208,10 @@ class TestExportSpec(TestSpec):
   - source: mylab.extensions.yaml
   version: 0.0.1
 """ % self.date.isoformat()  # noqa: E128
+            nsstr = file.read()
             self.assertEqual(nsstr, match_str)
 
     def test_missing_data_types(self):
+        """Test that calling export_spec on a namespace builder without data types raises a warning."""
         with self.assertWarnsWith(UserWarning, 'No data types specified. Exiting.'):
             export_spec(self.ns_builder, [], '.')
-
-    def test_missing_name(self):
-        self.ns_builder._NamespaceBuilder__ns_args['name'] = None
-        with self.assertRaisesWith(RuntimeError, 'Namespace name is required to export specs'):
-            export_spec(self.ns_builder, self.data_types, '.')
